@@ -170,8 +170,12 @@ GtkWidget *label_wrap(char *str) {
   GtkWidget *label = gtk_label_new(str);
   gtk_label_set_line_wrap_mode(GTK_LABEL(label), PANGO_WRAP_WORD);
   gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-#ifdef FREMANTLE
+#ifdef USE_MAEMO
+#if MAEMO_VERSION_MAJOR >= 5
   gtk_widget_set_size_request(label, 720, -1);
+#else
+  gtk_widget_set_size_request(label, 640, -1);
+#endif
 #else
   gtk_widget_set_size_request(label, 350, -1);
 #endif
@@ -282,46 +286,59 @@ GtkWidget *left_label(char *str) {
   return widget;
 }
 
+static void author_add(GtkWidget *box, char *str) {
+  gtk_box_pack_start(GTK_BOX(box), left_label(str), FALSE, FALSE, 0);
+}
+
 GtkWidget *authors_page_new(void) {
-  GtkWidget *ivbox, *vbox = gtk_vbox_new(FALSE, 0);
+  GtkWidget *ivbox, *vbox = gtk_vbox_new(FALSE, 16);
 
   /* -------------------------------------------- */
   ivbox = gtk_vbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(ivbox), left_label(_("Main developer:")),
-		     FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(ivbox), 
-		     left_label("Till Harbaum <till@harbaum.org>"),
-		     FALSE, FALSE, 0);
+  author_add(ivbox, _("Main developer:"));
+  author_add(ivbox, "Till Harbaum <till@harbaum.org>");
   gtk_box_pack_start(GTK_BOX(vbox), ivbox, TRUE, FALSE, 0);
 
   /* -------------------------------------------- */
   ivbox = gtk_vbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(ivbox), left_label(_("Original map widget by:")),
-		     FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(ivbox), 
-		     left_label("John Stowers <john.stowers@gmail.com>"),
-		     FALSE, FALSE, 0);
+  author_add(ivbox, _("Original map widget by:"));
+  author_add(ivbox, "John Stowers <john.stowers@gmail.com>");
   gtk_box_pack_start(GTK_BOX(vbox), ivbox, TRUE, FALSE, 0);
 
   /* -------------------------------------------- */
   ivbox = gtk_vbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(ivbox), left_label(_("GCVote service provided by:")),
-		     FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(ivbox), 
-		     left_label("Guido Wegener <guido.wegener@gmx.de>"),
-		     FALSE, FALSE, 0);
+  author_add(ivbox, _("GCVote service provided by:"));
+  author_add(ivbox, "Guido Wegener <guido.wegener@gmx.de>");
   gtk_box_pack_start(GTK_BOX(vbox), ivbox, TRUE, FALSE, 0);
 
   /* -------------------------------------------- */
   ivbox = gtk_vbox_new(FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(ivbox), left_label(_("Additional translations by:")),
-		     FALSE, FALSE, 0);
-  gtk_box_pack_start(GTK_BOX(ivbox), 
-		     left_label("Marko Vertainen <marko.vertainen@iki.fi>"),
-		     FALSE, FALSE, 0);
+  author_add(ivbox, _("Additional translations by:"));
+  author_add(ivbox, "Marko Vertainen <marko.vertainen@iki.fi>");
   gtk_box_pack_start(GTK_BOX(vbox), ivbox, TRUE, FALSE, 0);
 
-  return vbox;
+  /* -------------------------------------------- */
+  ivbox = gtk_vbox_new(FALSE, 0);
+  author_add(ivbox, _("Testers:"));
+  author_add(ivbox, "Uwe Koch <asys3@yahoo.com>");
+  author_add(ivbox, "Tanja Harbaum <tanja@harbaum.org>");
+  gtk_box_pack_start(GTK_BOX(vbox), ivbox, TRUE, FALSE, 0);
+
+#ifndef USE_PANNABLE_AREA
+  GtkWidget *scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
+  				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), 
+					vbox);
+  gtk_scrolled_window_set_shadow_type( GTK_SCROLLED_WINDOW(scrolled_window),
+				       GTK_SHADOW_IN);
+  return scrolled_window;
+#else
+  GtkWidget *pannable_area = hildon_pannable_area_new();
+  hildon_pannable_area_add_with_viewport(HILDON_PANNABLE_AREA(pannable_area), 
+					vbox);
+  return pannable_area;
+#endif
 }  
 
 GtkWidget *donate_page_new(appdata_t *appdata) {
@@ -381,7 +398,7 @@ void about_box(appdata_t *appdata) {
 	  GTK_WINDOW(appdata->window), GTK_DIALOG_MODAL,
           GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
 
-#ifdef FREMANTLE
+#ifdef USE_MAEMO
   gtk_window_set_default_size(GTK_WINDOW(dialog), 640, 480);
 #else
   gtk_window_set_default_size(GTK_WINDOW(dialog), 400, 200);
