@@ -68,9 +68,6 @@ static void bct_update(bct_priv_t *priv) {
     crumb_t *crumb = g_slist_last(priv->crumbs)->data;
     /* disable button now last in chain */
     gtk_widget_set_sensitive(crumb->button, FALSE);
-
-    //    GtkWidget *label =  gtk_bin_get_child(GTK_BIN(crumb->button));
-    //    gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_NONE);
   }
 
   /* disable back button if crumb trail has 1 item or less */
@@ -124,9 +121,6 @@ void bct_push_text(GtkWidget *bct, const gchar *text, gpointer id,
     crumb_t *crumb = g_slist_last(priv->crumbs)->data;
     gtk_widget_set_sensitive(crumb->button, TRUE);
     gtk_widget_set_sensitive(priv->back, TRUE);
-
-    //  GtkWidget *label =  gtk_bin_get_child(GTK_BIN(crumb->button));
-    //  gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
   }
 
   /* create a new crumb */
@@ -148,3 +142,9 @@ void bct_push_text(GtkWidget *bct, const gchar *text, gpointer id,
   priv->crumbs = g_slist_append(priv->crumbs, crumb);
 
 }
+
+void bct_pop(GtkWidget *bct) {
+  bct_priv_t *priv = g_object_get_data(G_OBJECT(bct), "priv");
+  g_assert(priv);
+  crumb_destroy(priv, g_slist_last(priv->crumbs)->data);
+}
