@@ -122,8 +122,8 @@ int dbus_mm_set_position(appdata_t *appdata, pos_t *pos) {
   osso_rpc_free_val(&retval);
 
   if(ret != OSSO_OK)
-    errorf(_("Unable to communicate with Maemo-Mapper. "
-	     "You need to have Maemo-Mapper installed "
+    errorf(_("Unable to communicate with Maemo Mapper. "
+	     "You need to have Maemo Mapper installed "
 	     "to use this feature."));
 
   return(ret == OSSO_OK)?0:1;
