@@ -38,10 +38,6 @@
 #define SAT_WIDTH 330
 #define SAT_HEIGHT 60
 
-/* http://www.gtk.org/tutorial1.2/gtk_tut-30.html */
-/* http://developer.gimp.org/api/2.0/gdk/gdk-Drawing-Primitives.html */
-
-
 static float rad2deg(float rad) {
   return fmodf(360.0 + (180.0/M_PI) * rad, 360.0);
 }
@@ -88,7 +84,7 @@ static void compass_draw(GtkWidget *widget, cache_context_t *context) {
   /* draw the locked/unlocked icon */
   gdk_draw_pixbuf(context->gotoc.compass_pixmap, 
 		  widget->style->fg_gc[GTK_STATE_NORMAL],
-		  icon_get(ICON_MISC, context->appdata->compass_locked?1:2), 
+		  icon_get(ICON_MISC, context->appdata->compass_locked?0:1), 
 		  0, 0, (width-diameter)/2 + diameter/32, 
 		  (height+diameter)/2 - 16 - diameter/32 , 16, 16,
 		  GDK_RGB_DITHER_NONE,0,0);
@@ -452,114 +448,6 @@ static gboolean goto_update(gpointer data) {
   return TRUE;   // fire again
 }
 
-static gint waypoint_changed_event(GtkWidget *widget, gpointer data ) {
-  cache_context_t *context = (cache_context_t*)data;
-  int wpt_idx = gtk_combo_box_get_active(GTK_COMBO_BOX(widget));
-
-  /* get position of selected waypoint */
-  if(wpt_idx == 0) 
-    context->gotoc.pos = context->appdata->geomath;
-  else if(wpt_idx == 1)
-    //    context->gotoc.pos = gpx_cache_pos(context->cache);
-    context->gotoc.pos = notes_get_pos(context);
-  else {
-    wpt_t *wpt = context->cache->wpt;
-    while(wpt_idx > 2) {
-      g_assert(wpt != NULL);
-      wpt = wpt->next;
-      wpt_idx--;
-    }
-    context->gotoc.pos = wpt->pos;
-  }
-
-  char str[32];
-  pos_lat_str(str, sizeof(str), context->gotoc.pos.lat);
-  gtk_label_set_text(GTK_LABEL(context->gotoc.lat_lbl), str);
-  pos_lon_str(str, sizeof(str), context->gotoc.pos.lon);
-  gtk_label_set_text(GTK_LABEL(context->gotoc.lon_lbl), str);
-
-  goto_update(context);
-
-  return FALSE;
-}
-
-static void manual_pos_update(cache_context_t *context) {
-  context->gotoc.pos.lat = context->appdata->manual_goto.lat;
-  context->gotoc.pos.lon = context->appdata->manual_goto.lon;
-
-  char str[32];
-  pos_lat_str(str, sizeof(str), context->gotoc.pos.lat);
-  gtk_label_set_text(GTK_LABEL(context->gotoc.lat_lbl), str);
-  pos_lon_str(str, sizeof(str), context->gotoc.pos.lon);
-  gtk_label_set_text(GTK_LABEL(context->gotoc.lon_lbl), str);
-
-  goto_update(context);
-}
-
-static void on_posedit_clicked(GtkWidget *button, gpointer data) {
-  cache_context_t *context = (cache_context_t*)data;
-
-  GtkWidget *dialog = gtk_dialog_new_with_buttons(_("Edit Position"),
-	  GTK_WINDOW(context->appdata->window), GTK_DIALOG_MODAL,
-          GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
-	  GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, NULL);
-
-  GtkWidget *lat, *lon, *label;
-  GtkWidget *table = gtk_table_new(2, 2, FALSE);
-
-  gtk_table_attach_defaults(GTK_TABLE(table),
-		   label = gtk_label_new(_("Latitude:")), 0, 1, 0, 1);
-  gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
-  gtk_table_attach_defaults(GTK_TABLE(table), 
-		    lat = lat_entry_new(context->appdata->manual_goto.lat),
-		    1, 2, 0, 1);
-
-  gtk_table_attach_defaults(GTK_TABLE(table),
-		    label = gtk_label_new(_("Longitude:")), 0, 1, 1, 2);
-  gtk_misc_set_alignment(GTK_MISC(label), 1.f, 0.5f);
-  gtk_table_attach_defaults(GTK_TABLE(table),
-		   lon = lon_entry_new(context->appdata->manual_goto.lon), 
-		   1, 2, 1, 2);
-
-  gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), table);
-
-  gtk_widget_show_all(dialog);
-
-  if(GTK_RESPONSE_ACCEPT == gtk_dialog_run(GTK_DIALOG(dialog))) {
-    if(isnan(lat_get(lat)) || isnan(lon_get(lon))) 
-      errorf(_("Ignoring invalid position"));
-    else {
-      context->appdata->manual_goto.lat = lat_get(lat);
-      context->appdata->manual_goto.lon = lon_get(lon);
-      
-      manual_pos_update(context);
-    }
-  }
-  gtk_widget_destroy(dialog);
-}
-
-static gint radio_changed_event(GtkWidget *widget, gpointer data ) {
-  cache_context_t *context = (cache_context_t*)data;
-
-  if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))) {
-    manual_pos_update(context);
-    gtk_widget_set_sensitive(context->gotoc.edit_but, TRUE);
-  } else {
-    waypoint_changed_event(context->gotoc.cbox, data);
-    gtk_widget_set_sensitive(context->gotoc.edit_but, FALSE);
-  }
-
-  return FALSE;
-}
-
-#ifdef ENABLE_MAEMO_MAPPER
-static void on_mm_button_clicked(GtkButton *button, gpointer data) {
-  cache_context_t *context = (cache_context_t*)data;
-
-  dbus_mm_set_position(context->appdata, &context->gotoc.pos);
-}
-#endif
-
 static gboolean compass_clicked_event(GtkWidget *widget, GdkEventButton *event,
 			    gpointer user_data) {
   cache_context_t *context = (cache_context_t*)user_data;
@@ -579,6 +467,16 @@ static gboolean compass_clicked_event(GtkWidget *widget, GdkEventButton *event,
   return FALSE;
 }
 
+static void pos_modified(GtkWidget *widget, gpointer data ) {
+  cache_context_t *context = (cache_context_t*)data;
+
+  /* extract position from entries */
+  context->gotoc.pos.lat = lat_entry_get(context->gotoc.lat_entry);
+  context->gotoc.pos.lon = lon_entry_get(context->gotoc.lon_entry);
+
+  goto_update(context);
+}
+
 GtkWidget *goto_cache(cache_context_t *context) {
   int i;
 
@@ -588,7 +486,7 @@ GtkWidget *goto_cache(cache_context_t *context) {
 
   context->gotoc.pos = gpx_cache_pos(context->cache);
 
-  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+  GtkWidget *hbox = gtk_hbox_new(FALSE, 32);
 
   context->gotoc.compass_area = gtk_drawing_area_new();
   gtk_drawing_area_size(GTK_DRAWING_AREA(context->gotoc.compass_area), 
@@ -607,88 +505,48 @@ GtkWidget *goto_cache(cache_context_t *context) {
   gtk_box_pack_start_defaults(GTK_BOX(hbox), context->gotoc.compass_area);
 
   GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
-  GtkWidget *table = gtk_table_new(8, 2, FALSE);
+  GtkWidget *table = gtk_table_new(5, 3, FALSE);
 
-  /* ---------- combo box containing waypoint names ------- */
-  GtkWidget *radio =  gtk_radio_button_new_with_label(NULL, _("Waypoint:"));
-  gtk_table_attach_defaults(GTK_TABLE(table), radio, 0,1,0,1);
+  /* ---------- destination coordinates ------- */
 
-  context->gotoc.cbox = gtk_combo_box_new_text();
-  gtk_combo_box_append_text(GTK_COMBO_BOX(context->gotoc.cbox), _("Geomath"));
-  gtk_combo_box_append_text(GTK_COMBO_BOX(context->gotoc.cbox), 
-			    context->cache->id);
-
-  wpt_t *wpt = context->cache->wpt;
-  while(wpt) {
-    gtk_combo_box_append_text(GTK_COMBO_BOX(context->gotoc.cbox), wpt->id);
-    wpt = wpt->next;
-  }
-  gtk_combo_box_set_active(GTK_COMBO_BOX(context->gotoc.cbox), 1);
-  gtk_signal_connect(GTK_OBJECT(context->gotoc.cbox), "changed",
-		     (GtkSignalFunc)waypoint_changed_event, context);
-  gtk_table_attach_defaults(GTK_TABLE(table), context->gotoc.cbox, 1,2,0,1);
-
-  /* -------------- manual entry field ------------------------- */
-  gtk_table_attach_defaults(GTK_TABLE(table), 
-	  gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio), 
-						      _("Manual:")), 0,1,1,2);
-  gtk_signal_connect(GTK_OBJECT(radio), "clicked",
-		     (GtkSignalFunc)radio_changed_event, context);
-
-  context->gotoc.edit_but = gtk_button_new_with_label(_("Edit"));
-  gtk_table_attach_defaults(GTK_TABLE(table), context->gotoc.edit_but, 1,2,1,2);
-  gtk_signal_connect(GTK_OBJECT(context->gotoc.edit_but), "clicked",
-		     (GtkSignalFunc)on_posedit_clicked, context);
-  gtk_widget_set_sensitive(context->gotoc.edit_but, FALSE);
-
-#if 0
-  context->gotoc.man_lat = gtk_entry_new();
-  gtk_table_attach_defaults(GTK_TABLE(table), context->gotoc.man_lat, 1,2,1,2);
-  pos_lat_str(str, sizeof(str), context->appdata->manual_goto.lat);
-  gtk_entry_set_text(GTK_ENTRY(context->gotoc.man_lat), str);
-  textbox_disable(context->gotoc.man_lat);
-  context->gotoc.man_lon = gtk_entry_new();
-  gtk_table_attach_defaults(GTK_TABLE(table), context->gotoc.man_lon, 1,2,2,3);
-  pos_lon_str(str, sizeof(str), context->appdata->manual_goto.lon);
-  gtk_entry_set_text(GTK_ENTRY(context->gotoc.man_lon), str);  
-  textbox_disable(context->gotoc.man_lon);
-  
-  gtk_signal_connect(GTK_OBJECT(context->gotoc.man_lat), "changed",
-		     (GtkSignalFunc)entry_changed_event, context);
-  gtk_signal_connect(GTK_OBJECT(context->gotoc.man_lon), "changed",
-		     (GtkSignalFunc)entry_changed_event, context);
-#endif
-
-  gtk_table_set_row_spacing(GTK_TABLE(table), 2, 16);
-
-  /* -------------- waypoint coordinates ------------------------- */
   /* SIZE_SMALL doesn't work here as setting the label returns to normal */
-  context->gotoc.lat_lbl = pos_lat(context->gotoc.pos.lat, SIZE_NORMAL, 
-			       STRIKETHROUGH_NONE);
-  gtk_table_attach_defaults(GTK_TABLE(table), context->gotoc.lat_lbl,0,1,3,4);
-  context->gotoc.lon_lbl = pos_lon(context->gotoc.pos.lon, SIZE_NORMAL, 
-			       STRIKETHROUGH_NONE);
-  gtk_table_attach_defaults(GTK_TABLE(table), context->gotoc.lon_lbl,1,2,3,4);
+  gtk_table_attach_defaults(GTK_TABLE(table), left_label_new(_("Latitude:")), 0,1,0,1);
+  context->gotoc.lat_entry = lat_entry_new(context->gotoc.pos.lat);
+  gtk_table_attach_defaults(GTK_TABLE(table), context->gotoc.lat_entry, 1,2,0,1);
+  gtk_table_attach_defaults(GTK_TABLE(table), left_label_new(_("Longitude:")), 0,1,1,2);
+  context->gotoc.lon_entry = lon_entry_new(context->gotoc.pos.lon);
+  gtk_table_attach_defaults(GTK_TABLE(table), context->gotoc.lon_entry, 1,2,1,2);
+
+  g_signal_connect(G_OBJECT(context->gotoc.lat_entry), "changed", 
+		   G_CALLBACK(pos_modified), context);
+  g_signal_connect(G_OBJECT(context->gotoc.lon_entry), "changed", 
+		   G_CALLBACK(pos_modified), context);
+
+  gtk_table_attach_defaults(GTK_TABLE(table), preset_coordinate_picker(context->appdata, 
+	       context->gotoc.lat_entry, context->gotoc.lon_entry), 2,3,0,1);
+
+  gtk_table_attach_defaults(GTK_TABLE(table), goto_coordinate(context->appdata, 
+		    context->gotoc.lat_entry, context->gotoc.lon_entry), 2,3,1,2);
+
+  gtk_table_set_row_spacing(GTK_TABLE(table), 1, 16);
+  gtk_table_set_col_spacing(GTK_TABLE(table), 0, 16);
 
   /* -------------- distance label ------------------------- */
-  gtk_table_attach_defaults(GTK_TABLE(table), 
-	    gtk_label_new(_("Distance:")), 0,1,4,5);
+  gtk_table_attach_defaults(GTK_TABLE(table), left_label_new(_("Distance:")), 0,1,2,3);
   gtk_table_attach_defaults(GTK_TABLE(table),
-	    (context->gotoc.distance_label = gtk_label_new("-----")), 1,2,4,5);
+	    (context->gotoc.distance_label = gtk_label_new("-----")), 1,2,2,3);
 
   /* -------------- bearing label ------------------------- */
-  gtk_table_attach_defaults(GTK_TABLE(table), 
-	    gtk_label_new(_("Bearing:")), 0,1,5,6);
+  gtk_table_attach_defaults(GTK_TABLE(table), left_label_new(_("Bearing:")), 0,1,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table),
-	    (context->gotoc.bearing_label = gtk_label_new("-----")), 1,2,5,6);
+	    (context->gotoc.bearing_label = gtk_label_new("-----")), 1,2,3,4);
 
   /* -------------- error label ------------------------- */
-  gtk_table_attach_defaults(GTK_TABLE(table), 
-	    gtk_label_new(_("Est. error:")), 0,1,6,7);
+  gtk_table_attach_defaults(GTK_TABLE(table), left_label_new(_("Est. error:")), 0,1,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table),
-	    (context->gotoc.eph_label = gtk_label_new("-----")), 1,2,6,7);
+	    (context->gotoc.eph_label = gtk_label_new("-----")), 1,2,4,5);
 
-  gtk_table_set_row_spacing(GTK_TABLE(table), 6, 16);
+  gtk_table_set_row_spacing(GTK_TABLE(table), 4, 16);
 
   /* -------------- sat view box ------------------------- */
   GtkWidget *ihbox = gtk_hbox_new(FALSE,0);
@@ -706,29 +564,30 @@ GtkWidget *goto_cache(cache_context_t *context) {
 
   gtk_box_pack_start(GTK_BOX(ihbox), context->gotoc.sat_area, 1,0,0);
 
-#ifdef ENABLE_MAEMO_MAPPER
-  GtkWidget *mm_button = gtk_button_new();
-  gtk_button_set_image(GTK_BUTTON(mm_button), icon_get_widget(ICON_MISC, 0));
-  gtk_signal_connect(GTK_OBJECT(mm_button), "clicked",
-		     (GtkSignalFunc)on_mm_button_clicked, context);
-  gtk_box_pack_start(GTK_BOX(ihbox), mm_button, 1,0,0);
-#endif
-  gtk_table_attach_defaults(GTK_TABLE(table), ihbox, 0,2,7,8);
+  gtk_table_attach_defaults(GTK_TABLE(table), ihbox, 0,3,5,6);
 
   /* ------------------------------------------------------- */
 
-  gtk_box_pack_start(GTK_BOX(vbox), table, 1,0,0);
-  gtk_box_pack_start(GTK_BOX(hbox), vbox, 1,0,0);
+  gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
  
-  context->gotoc.handler_id = gtk_timeout_add(UPDATE_MS, 
-					     goto_update, context);
+  context->gotoc.handler_id = 
+    gtk_timeout_add(UPDATE_MS, goto_update, context);
 
   return hbox;
 }
 
 void goto_coordinate_update(cache_context_t *context) {
+  static pos_t pos = { 0.0, 0.0 };
+
   if(!context->notes.modified)
     return;
-  
-  waypoint_changed_event(context->gotoc.cbox, context);
+
+  pos_t npos = notes_get_pos(context);
+  if(pos_differ(&npos, &pos)) {
+    pos = npos;
+
+    lat_entry_set(context->gotoc.lat_entry, npos.lat);
+    lon_entry_set(context->gotoc.lon_entry, npos.lon);
+  }
 }
