@@ -443,6 +443,9 @@ void gps_init(appdata_t *appdata) {
     g_thread_create(gps_thread, appdata, FALSE, NULL);
 }
 
+void gps_change_state(appdata_t *appdata) {
+}
+
 void gps_release(appdata_t *appdata) { 
   gps_unregister_all(appdata);
 #ifdef USE_MAEMO
@@ -517,11 +520,41 @@ void gps_init(appdata_t *appdata) {
   if(gps_state->control
 #if MAEMO_VERSION_MAJOR < 5
      && gps_state->control->can_control
+     && appdata->use_gps
 #endif
      ) {
 
     printf("gps: Having control over GPSD and GPS is to be enabled, starting it\n");
     location_gpsd_control_start(gps_state->control);
+    gps_state->in_use = TRUE;
+  } else
+    gps_state->in_use = FALSE;
+}
+
+void gps_change_state(appdata_t *appdata) {
+  gps_state_t *gps_state = appdata->gps_state;
+
+  /* only do something if gps actually changes state */
+  if(( appdata->use_gps &&  gps_state->in_use) ||
+     (!appdata->use_gps && !gps_state->in_use))
+    return;
+
+  if(gps_state->control
+#if MAEMO_VERSION_MAJOR < 5
+     && gps_state->control->can_control
+#endif
+     ) {
+    printf("gps: Having control over GPSD and GPS. ");
+
+    if(appdata->use_gps) {
+      printf("Starting it!\n");
+      location_gpsd_control_start(gps_state->control);
+      gps_state->in_use = TRUE;
+    } else {
+      printf("Stopping it!\n");
+      location_gpsd_control_stop(gps_state->control);
+      gps_state->in_use = FALSE;
+    }
   }
 }
 
