@@ -118,6 +118,7 @@ typedef struct gps_state {
 #else
   LocationGPSDevice *device;
   LocationGPSDControl *control;
+  gboolean in_use;
   guint idd_changed;
   int fields;
   
@@ -135,6 +136,7 @@ pos_t *gps_get_pos(appdata_t *appdata);
 float gps_get_heading(appdata_t *appdata);
 float gps_get_eph(appdata_t *appdata);
 gps_sat_t *gps_get_sats(appdata_t *appdata);
+void gps_change_state(appdata_t *appdata);
 
 void *gps_register_callback(appdata_t *appdata, gps_cb cb, gpointer data);
 void gps_unregister_callback(appdata_t *appdata, void *cb);
