@@ -435,7 +435,10 @@ static void xml_get_date(xmlTextReaderPtr reader, int *year, int *month, int *da
   }
 }
 
-static log_t *process_gpx_wpt_gc_logs_log(xmlTextReaderPtr reader) {
+static log_t *process_gpx_wpt_gc_logs_log(xmlTextReaderPtr reader, 
+					  char *username, cache_t *cache) {
+
+  gboolean by_user = FALSE;
 
   if(xmlTextReaderIsEmptyElement(reader))
     return NULL;
@@ -469,6 +472,10 @@ static log_t *process_gpx_wpt_gc_logs_log(xmlTextReaderPtr reader) {
 	    log->finder = g_new0(user_t, 1);
 	    log->finder->name = process_text(reader);
 	    log->finder->id = xml_get_prop_id(reader);
+
+	    /* is this a log by us? */
+	    if(username && !strcasecmp(username, log->finder->name))
+	      by_user = TRUE;
 	  }
 	} else if(strcasecmp(name, "text") == 0) {
 	  if(!log->text) log->text = process_text(reader);
@@ -479,6 +486,9 @@ static log_t *process_gpx_wpt_gc_logs_log(xmlTextReaderPtr reader) {
       break;
       
     case XML_READER_TYPE_END_ELEMENT:
+      if(by_user && log->type == LOG_TYPE_FOUND) 
+	cache->found = TRUE;
+
       /* end element must be for the current element */
       g_assert(xmlTextReaderDepth(reader) == depth);
       return log;
@@ -494,7 +504,8 @@ static log_t *process_gpx_wpt_gc_logs_log(xmlTextReaderPtr reader) {
   return NULL;
 }
 
-static log_t *process_gpx_wpt_gc_logs(xmlTextReaderPtr reader) {
+static log_t *process_gpx_wpt_gc_logs(xmlTextReaderPtr reader, char *username,
+				      cache_t *cache) {
   log_t *log_chain = NULL;
 
   if(xmlTextReaderIsEmptyElement(reader))
@@ -512,7 +523,7 @@ static log_t *process_gpx_wpt_gc_logs(xmlTextReaderPtr reader) {
       if(strrchr(name, ':')) name = strrchr(name, ':')+1;
       if(name) {
 	if(strcasecmp(name, "log") == 0) {
-	  log_t *log = process_gpx_wpt_gc_logs_log(reader);
+	  log_t *log = process_gpx_wpt_gc_logs_log(reader, username, cache);
 	  if(log) {
 	    /* add log to chain */
 	    log_t **cur = &log_chain;
@@ -650,7 +661,8 @@ static tb_t *process_gpx_wpt_gc_tbs(xmlTextReaderPtr reader) {
   return NULL;
 }
 
-static void process_gpx_wpt_gc(xmlTextReaderPtr reader, cache_t *cache) {
+static void process_gpx_wpt_gc(xmlTextReaderPtr reader, 
+			       cache_t *cache, char *username) {
   cache->available = xml_prop_is(reader, "available", "true", TRUE);
   cache->archived = xml_prop_is(reader, "archived", "true", FALSE);
 
@@ -675,6 +687,9 @@ static void process_gpx_wpt_gc(xmlTextReaderPtr reader, cache_t *cache) {
 	    cache->owner = g_new0(user_t, 1);
 	    cache->owner->name = process_text(reader);
 	    cache->owner->id = xml_get_prop_id(reader);
+	    
+	    if(username && !strcasecmp(cache->owner->name, username))
+	      cache->mine = TRUE;
 	  }
 	} else if(strcasecmp(name, "type") == 0) {
 	  cache->type = xml_str_search(reader, cache_type_str, 
@@ -711,7 +726,8 @@ static void process_gpx_wpt_gc(xmlTextReaderPtr reader, cache_t *cache) {
 	} else if(strcasecmp(name, "terrain") == 0) {
 	  cache->terrain = xml_float(reader, 0.0);
 	} else if(strcasecmp(name, "logs") == 0) {
-	  if(!cache->log) cache->log = process_gpx_wpt_gc_logs(reader);
+	  if(!cache->log) cache->log = 
+		    process_gpx_wpt_gc_logs(reader, username, cache);
 	} else if(strcasecmp(name, "travelbugs") == 0) {
 	  if(!cache->tb) cache->tb = process_gpx_wpt_gc_tbs(reader);
 	} else {
@@ -737,7 +753,7 @@ static void process_gpx_wpt_gc(xmlTextReaderPtr reader, cache_t *cache) {
 
 /* parse waypoint entry */
 static cache_t *process_gpx_wpt(xmlTextReaderPtr reader,  gpx_dialog_t *dialog,
-				gpx_t *gpx) {
+				gpx_t *gpx, char *username) {
   char *cmt = NULL, *desc = NULL;
   char *sym = NULL;
 
@@ -777,7 +793,7 @@ static cache_t *process_gpx_wpt(xmlTextReaderPtr reader,  gpx_dialog_t *dialog,
 	  if(!cache->url) cache->url = process_text(reader);
 	} else if((strcasecmp(name, "cache") == 0) || 
 		  (strcasecmp(name, "geocache") == 0)) {
-	  process_gpx_wpt_gc(reader, cache);
+	  process_gpx_wpt_gc(reader, cache, username);
 
 	  /* the following are used if the current entry is a waypoint */
 	} else if(strcasecmp(name, "cmt") == 0) {
@@ -882,7 +898,7 @@ static cache_t *process_gpx_wpt(xmlTextReaderPtr reader,  gpx_dialog_t *dialog,
 }
 
 static gboolean process_gpx(xmlTextReaderPtr reader, gpx_dialog_t *dialog, 
-			    gpx_t *gpx) {
+			    gpx_t *gpx, char *username) {
 
   /* no attributes of interest */
 
@@ -912,7 +928,7 @@ static gboolean process_gpx(xmlTextReaderPtr reader, gpx_dialog_t *dialog,
 			 (strcasecmp(name, "date") == 0))) {
 	xml_get_date(reader, &gpx->year, &gpx->month, &gpx->day);
       } else if(name && strcasecmp(name, "wpt") == 0) {
-	*cache = process_gpx_wpt(reader, dialog, gpx);
+	*cache = process_gpx_wpt(reader, dialog, gpx, username);
 	if(*cache) cache = &(*cache)->next;
       } else {
 	//	printf("something unknown (%s) found\n", name);
@@ -1039,7 +1055,7 @@ static void process_loc(xmlTextReaderPtr reader, gpx_dialog_t *dialog,
 }
 
 static gpx_t *process_root(xmlTextReaderPtr reader, gpx_dialog_t *dialog, 
-			   char *fname, gpx_t *in) {
+			   char *fname, gpx_t *in, char *username) {
 
   /* no gpx entry given, create a new one */
   gpx_t *gpx = NULL;
@@ -1058,7 +1074,7 @@ static gpx_t *process_root(xmlTextReaderPtr reader, gpx_dialog_t *dialog,
       g_assert(xmlTextReaderDepth(reader) == 0);
       char *name = (char*)xmlTextReaderConstName(reader);
       if(name && strcasecmp(name, "gpx") == 0) {
-	process_gpx(reader, dialog, gpx);
+	process_gpx(reader, dialog, gpx, username);
       } else if(name && strcasecmp(name, "loc") == 0) {
 	process_loc(reader, dialog, gpx);
       } else {
@@ -1094,7 +1110,8 @@ static gpx_t *process_root(xmlTextReaderPtr reader, gpx_dialog_t *dialog,
   return gpx;
 }
 
-static gpx_t *gpx_parse_file(gpx_dialog_t *dialog, char *filename) {
+static gpx_t *gpx_parse_file(gpx_dialog_t *dialog, 
+			     char *filename, char *username) {
   gpx_t *gpx = NULL;
 
   LIBXML_TEST_VERSION;
@@ -1103,7 +1120,7 @@ static gpx_t *gpx_parse_file(gpx_dialog_t *dialog, char *filename) {
 
   xmlTextReaderPtr reader = xmlReaderForFile(filename, NULL, 0);
   if (reader != NULL) {
-    gpx = process_root(reader, dialog, filename, NULL);
+    gpx = process_root(reader, dialog, filename, NULL, username);
     xmlFreeTextReader(reader);
   } else {
     fprintf(stderr, "Unable to open %s\n", filename);
@@ -1119,7 +1136,7 @@ static gpx_t *gpx_parse_file(gpx_dialog_t *dialog, char *filename) {
     if(g_file_test(wpts_name,  G_FILE_TEST_EXISTS)) {
       xmlTextReaderPtr reader = xmlReaderForFile(wpts_name, NULL, 0);
       if (reader != NULL) {
-	gpx = process_root(reader, dialog, wpts_name, gpx);
+	gpx = process_root(reader, dialog, wpts_name, gpx, username);
 	xmlFreeTextReader(reader);
       } else {
 	fprintf(stderr, "Unable to open %s\n", filename);
@@ -1132,7 +1149,7 @@ static gpx_t *gpx_parse_file(gpx_dialog_t *dialog, char *filename) {
 
 static gpx_t *decompress_file(unzFile file, gpx_dialog_t *dialog, 
 			      char *name, char *filename,
-			      gpx_t *gpx_in) {
+			      gpx_t *gpx_in, char *username) {
   unz_file_info info;
   gpx_t *gpx = NULL;
   
@@ -1176,7 +1193,7 @@ static gpx_t *decompress_file(unzFile file, gpx_dialog_t *dialog,
     xmlReaderForMemory(buffer, info.uncompressed_size, 
 		       NULL, NULL, 0);
   if (reader != NULL) {
-    gpx = process_root(reader, dialog, filename, gpx_in);
+    gpx = process_root(reader, dialog, filename, gpx_in, username);
     xmlFreeTextReader(reader);
   } else {
     fprintf(stderr, "Unable to open %s\n", filename);
@@ -1187,7 +1204,8 @@ static gpx_t *decompress_file(unzFile file, gpx_dialog_t *dialog,
   return gpx;
 }
 
-static gpx_t *decompress_zip(gpx_dialog_t *dialog, char *filename) {
+static gpx_t *decompress_zip(gpx_dialog_t *dialog, 
+			     char *filename, char *username) {
   char *gpx_name, *fbase;
   gpx_t *gpx = NULL;
 
@@ -1211,21 +1229,21 @@ static gpx_t *decompress_zip(gpx_dialog_t *dialog, char *filename) {
   strcpy(gpx_name+strlen(gpx_name)-4, ".gpx");
   printf("gpx file name is %s\n", gpx_name);
 
-  gpx = decompress_file(file, dialog, gpx_name, filename, NULL);
+  gpx = decompress_file(file, dialog, gpx_name, filename, NULL, username);
 
   /* try to open -wpts.gpx file inside */
   strcpy(gpx_name, fbase);
   strcpy(gpx_name+strlen(gpx_name)-4, "-wpts.gpx");
   printf("gpx wpts file name is %s\n", gpx_name);
 
-  gpx = decompress_file(file, dialog, gpx_name, filename, gpx);
+  gpx = decompress_file(file, dialog, gpx_name, filename, gpx, username);
 
   unzClose(file);
   free(gpx_name);
   return gpx;
 }
 
-gpx_t *gpx_parse(gpx_dialog_t *dialog, char *filename) {
+gpx_t *gpx_parse(gpx_dialog_t *dialog, char *filename, char *username) {
   gpx_t *gpx = NULL;
 
   /* show busy dialog */
@@ -1235,15 +1253,15 @@ gpx_t *gpx_parse(gpx_dialog_t *dialog, char *filename) {
      !strcasecmp(filename+strlen(filename)-4, ".zip")) {
     printf("trying to load a zip file!\n");
 
-    gpx = decompress_zip(dialog, filename);
+    gpx = decompress_zip(dialog, filename, username);
   } else 
-    gpx = gpx_parse_file(dialog, filename);
+    gpx = gpx_parse_file(dialog, filename, username);
 
   return gpx;
 }
 
 /* scan entire directory */
-gpx_t *gpx_parse_dir(gpx_dialog_t *dialog, char *dirname) {
+gpx_t *gpx_parse_dir(gpx_dialog_t *dialog, char *dirname, char *username) {
   GnomeVFSResult result;
   GnomeVFSDirectoryHandle *handle;
   GnomeVFSFileInfo *finfo = gnome_vfs_file_info_new();;
@@ -1280,7 +1298,7 @@ gpx_t *gpx_parse_dir(gpx_dialog_t *dialog, char *dirname) {
 
 	xmlTextReaderPtr reader = xmlReaderForFile(filename, NULL, 0);
 	if (reader != NULL) {
-	  gpx = process_root(reader, dialog, filename, gpx);
+	  gpx = process_root(reader, dialog, filename, gpx, username);
 	  xmlFreeTextReader(reader);
 	} else {
 	  fprintf(stderr, "Unable to open %s\n", filename);
