@@ -120,7 +120,6 @@ typedef struct {
 
   int gpxlist_items;
   int cachelist_items;
-  guint cachelist_handler_id; /* update timer */
   gboolean cachelist_hide_found;
 
   /* geotext related */
@@ -231,7 +230,9 @@ typedef struct {
     int flags;
   } gt;
 
+#ifndef USE_STACKABLE_WINDOW
   GtkWidget *vbox;    
+#endif
 } appdata_t;
 
 /* all includes that rely on appdata_t go here ... */
