@@ -256,11 +256,11 @@ typedef struct {
   appdata_t *appdata;
   GtkTreePath *path;  
   gboolean done;
-} cachelist_expose_t;
+} cachelist_context_t;
 
 static gboolean cachelist_expose(GtkWidget *widget, GdkEventExpose *event,
 				 gpointer user_data) {
-  cachelist_expose_t *ce = (cachelist_expose_t*)user_data;
+  cachelist_context_t *ce = (cachelist_context_t*)user_data;
 
   if(event->type == GDK_EXPOSE) {
     if(ce->path && !ce->done) {
@@ -275,14 +275,22 @@ static gboolean cachelist_expose(GtkWidget *widget, GdkEventExpose *event,
 }
 
 static void cachelist_destroy(GtkWidget *widget, gpointer user_data) {
-  cachelist_expose_t *ce = (cachelist_expose_t*)user_data;
 
-  printf("cachelist timer removed\n");
-  g_assert(ce->appdata->cachelist_handler_id);
-  gtk_timeout_remove(ce->appdata->cachelist_handler_id);
-  ce->appdata->cachelist_handler_id = 0;
+  guint handler_id = 
+    (guint)g_object_get_data(G_OBJECT(user_data), "handler_id");
 
-  free(user_data);
+  if(handler_id) {
+    gtk_timeout_remove(handler_id);
+    g_object_set_data(G_OBJECT(user_data), "handler_id", NULL);
+
+    printf("cachelist timer removed\n");
+  }
+
+  gpointer *ce = 
+    g_object_get_data(G_OBJECT(user_data), "ce");
+  g_assert(ce);
+
+  free(ce);
 }
 
 #define CACHELIST_UPDATE_TIMEOUT (30000)
@@ -291,6 +299,8 @@ static GtkWidget *cachelist_create(appdata_t *appdata, gpx_t *gpx,
 				   cache_t *sel_cache);
 
 void cachelist_redraw(appdata_t *appdata) {
+  printf("redrawing cachelist\n");
+
   if(!appdata->cur_view) {
     printf("cachelist redraw: no active view\n");
     return;
@@ -306,11 +316,11 @@ void cachelist_redraw(appdata_t *appdata) {
   }
 
   if(redraw) {
-    GtkWidget *container = appdata->vbox;
-
 #ifdef USE_STACKABLE_WINDOW
     HildonWindowStack *stack = hildon_window_stack_get_default();
-    container = hildon_window_stack_peek(stack);
+    GtkWidget *container = hildon_window_stack_peek(stack);
+#else
+    GtkWidget *container = appdata->vbox;
 #endif
 
     gtk_container_remove(GTK_CONTAINER(container), appdata->cur_view);
@@ -326,11 +336,10 @@ void cachelist_redraw(appdata_t *appdata) {
     }
 
 #ifdef USE_STACKABLE_WINDOW
-    if(container != appdata->vbox)
-      gtk_container_add(GTK_CONTAINER(container), appdata->cur_view); 
-    else
+    gtk_container_add(GTK_CONTAINER(container), appdata->cur_view); 
+#else
+    gtk_box_pack_start_defaults(GTK_BOX(container), appdata->cur_view);
 #endif
-      gtk_box_pack_start_defaults(GTK_BOX(container), appdata->cur_view);
     
     gtk_widget_show_all(container);
   }
@@ -338,10 +347,19 @@ void cachelist_redraw(appdata_t *appdata) {
 
 
 static gboolean cachelist_update(gpointer data) {
+  appdata_t *appdata = 
+    (appdata_t*)g_object_get_data(G_OBJECT(data), "appdata");
+  g_assert(appdata);
 
   printf("cachelist timer fired!\n");
 
-  appdata_t *appdata = (appdata_t*)data;
+  /* check if the widget the timer fired for is the currently */
+  /* visible one (if a search result is being shown, a cachlist */
+  /* may also be present below it) */
+  if(appdata->cur_view != data) {
+    printf("-> widget is not the one currently on top, don't redraw\n");
+    return TRUE;
+  }
 
   if(appdata->cur_cache)
     return TRUE;
@@ -354,30 +372,73 @@ static gboolean cachelist_update(gpointer data) {
 
   if(appdata->cachelist_update)
     cachelist_redraw(appdata);
+  else
+    printf("update disabled\n");
 
   return TRUE;
 }
 
-static void cachelist_timer_reset(appdata_t *appdata) {
+static void cachelist_timer_reset(GtkWidget *widget) {
+  guint handler_id = 
+    (guint)g_object_get_data(G_OBJECT(widget), "handler_id");
+  g_assert(handler_id);
+
+  appdata_t *appdata = 
+    (appdata_t*)g_object_get_data(G_OBJECT(widget), "appdata");
+  g_assert(appdata);
+
   printf("cachelist timer reset\n");
-  g_assert(appdata->cachelist_handler_id);
-  gtk_timeout_remove(appdata->cachelist_handler_id);
-  appdata->cachelist_handler_id = 
-    gtk_timeout_add(CACHELIST_UPDATE_TIMEOUT, cachelist_update, appdata);  
+  gtk_timeout_remove(handler_id);
+  g_object_set_data(G_OBJECT(widget), "handler_id", (gpointer)
+    gtk_timeout_add(CACHELIST_UPDATE_TIMEOUT, cachelist_update, widget));  
 }
 
 static gboolean cachelist_update_reset0(GtkWidget *widget,
 					GdkEventButton *event,
 					gpointer user_data) {
-  cachelist_timer_reset((appdata_t*)user_data);
+  cachelist_timer_reset(GTK_WIDGET(user_data));
   return FALSE;
 }
 
 static void cachelist_update_reset1(GtkAdjustment *adj,
 				    gpointer user_data) {
-  cachelist_timer_reset((appdata_t*)user_data);
+  cachelist_timer_reset(GTK_WIDGET(user_data));
 }
 
+static gboolean on_cachelist_focus_in(GtkWidget *widget, GdkEventFocus *event,
+				      gpointer data) {
+  
+
+  /* we don't want a runnign timer yet */
+  if(!g_object_get_data(G_OBJECT(data), "handler_id")) {
+    printf("focus received: restarting cachelist timer\n");
+
+    appdata_t *appdata = 
+      (appdata_t*)g_object_get_data(G_OBJECT(data), "appdata");
+    g_assert(appdata);
+
+    g_object_set_data(G_OBJECT(data), "handler_id", (gpointer)
+      gtk_timeout_add(CACHELIST_UPDATE_TIMEOUT, cachelist_update, data));  
+  }
+
+  return FALSE;
+}  
+
+static gboolean on_cachelist_focus_out(GtkWidget *widget, GdkEventFocus *event,
+				       gpointer data) {
+
+  guint handler_id = 
+    (guint)g_object_get_data(G_OBJECT(data), "handler_id");
+  g_assert(handler_id);
+
+  gtk_timeout_remove(handler_id);
+  g_object_set_data(G_OBJECT(data), "handler_id", NULL);
+
+  printf("focus lost: cachelist timer removed\n");
+
+  return FALSE;
+}  
+
 static GtkWidget *cachelist_create(appdata_t *appdata, gpx_t *gpx,
 				   cache_t *sel_cache) {
   GtkCellRenderer *renderer;
@@ -523,53 +584,65 @@ static GtkWidget *cachelist_create(appdata_t *appdata, gpx_t *gpx,
   g_signal_connect(view, "row-activated", 
 		   (GCallback)cachelist_view_onRowActivated, appdata);
 
-  cachelist_expose_t *ce = malloc(sizeof(cachelist_expose_t));
+  cachelist_context_t *ce = g_new0(cachelist_context_t, 1);
   ce->appdata = appdata;
   ce->path = path;
   ce->done = FALSE;
 
   g_signal_connect(view, "expose-event", 
 		   (GCallback)cachelist_expose, ce);
-  g_signal_connect(view, "destroy",
-		   (GCallback)cachelist_destroy, ce);
 
   /* put this inside a scrolled view */
 #ifndef USE_PANNABLE_AREA
-  GtkWidget *scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
+  GtkWidget *container = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(container), 
 				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(GTK_CONTAINER(scrolled_window), view);
+  gtk_container_add(GTK_CONTAINER(container), view);
 #else
-  GtkWidget *pannable_area = hildon_pannable_area_new();
-  
-  gtk_container_add(GTK_CONTAINER(pannable_area), view);
+  GtkWidget *container = hildon_pannable_area_new();
+  gtk_container_add(GTK_CONTAINER(container), view);
 #endif
 
-  /* add a timer for automatic update */
-  g_assert(!appdata->cachelist_handler_id);
-  appdata->cachelist_handler_id = 
-    gtk_timeout_add(CACHELIST_UPDATE_TIMEOUT, cachelist_update, appdata);
+  g_signal_connect(view, "destroy",
+		   (GCallback)cachelist_destroy, container);
 
   /* update timer is being reset if the user scrolls or selects */
   g_signal_connect(view, "button-press-event", 
-		   (GCallback)cachelist_update_reset0, appdata);
+		   (GCallback)cachelist_update_reset0, container);
+
+  /* add a timer for automatic update */
+  g_object_set_data(G_OBJECT(container), "handler_id", (gpointer)
+    gtk_timeout_add(CACHELIST_UPDATE_TIMEOUT, cachelist_update, container));  
+
+  printf("cachelist timer created\n");
+
+  g_object_set_data(G_OBJECT(container), "appdata", (gpointer)appdata);
+ 
+  g_object_set_data(G_OBJECT(container), "ce", (gpointer)ce);
+
+  /* the timer is removed and re-enabled on every focus change event */
+  /* for the main top window */
+  //  GtkWidget *root = gtk_widget_get_toplevel(GTK_WIDGET(button)))
+
+  g_signal_connect(G_OBJECT(view), "focus-in-event", 
+		   G_CALLBACK(on_cachelist_focus_in), container);
+
+  g_signal_connect(G_OBJECT(view), "focus-out-event", 
+		   G_CALLBACK(on_cachelist_focus_out), container);
 
 #ifndef USE_PANNABLE_AREA
   g_signal_connect(gtk_scrolled_window_get_vadjustment(
-		       GTK_SCROLLED_WINDOW(scrolled_window)), 
-			   "value-changed", 
-			   (GCallback)cachelist_update_reset1, appdata);
-
-  return scrolled_window;
+	       GTK_SCROLLED_WINDOW(container)), 
+	       "value-changed", 
+	       (GCallback)cachelist_update_reset1, container);
 #else
   g_signal_connect(hildon_pannable_area_get_vadjustment(
-		       HILDON_PANNABLE_AREA(pannable_area)), 
-			   "value-changed", 
-			   (GCallback)cachelist_update_reset1, appdata);
+	       HILDON_PANNABLE_AREA(container)), 
+	       "value-changed", 
+	       (GCallback)cachelist_update_reset1, container);
 
-
-  return pannable_area;
 #endif
+  return container;
 }
 
 #ifndef USE_MAEMO
@@ -821,11 +894,6 @@ static void gpxlist_view_onRowActivated(GtkTreeView *treeview,
 
       GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
 
-      gtk_box_pack_start(GTK_BOX(hbox),
-	 gtk_image_new_from_stock( GTK_STOCK_DIALOG_QUESTION, 
-				   GTK_ICON_SIZE_DIALOG),
-			 FALSE, FALSE, 0);
-
       GtkWidget *label = gtk_label_new(
 		  _("Do you want to close this entry only or do "
 		    "you want to remove it completely from the list?"));
@@ -975,6 +1043,8 @@ static GtkWidget *gpxlist_create_view_and_model(appdata_t *appdata,
 
   appdata->gpxview = gtk_tree_view_new ();
 
+  printf("building gpx list, items = %d\n", appdata->gpxlist_items);
+
   GtkTreeSelection *selection = 
     gtk_tree_view_get_selection(GTK_TREE_VIEW(appdata->gpxview));
 #ifndef USE_PANNABLE_AREA
@@ -1417,16 +1487,9 @@ cb_menu_search(GtkWidget *window, gpointer data) {
   g_signal_connect(G_OBJECT(context.in_finds), "toggled",
 		   G_CALLBACK(callback_finds_toggled), &context);
 
-#ifndef USE_MAEMO
-  GtkObject *adj = gtk_adjustment_new(appdata->search_days, 0, 99, 1, 10, 10);
-  context.spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
-#else
-  context.spinner = hildon_number_editor_new(0, 99);
-  hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(context.spinner), 
-				 appdata->search_days);
-#endif
+  context.spinner = number_editor_new(appdata->search_days, 0, 99);
   gtk_box_pack_start_defaults(GTK_BOX(hbox), context.spinner);
-
+			      
   gtk_box_pack_start_defaults(GTK_BOX(hbox), gtk_label_new(_("days")));
 
   gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox);
@@ -1444,13 +1507,7 @@ cb_menu_search(GtkWidget *window, gpointer data) {
     if(strlen(p) > 0)
       appdata->search_str = strdup(p);
 
-#ifndef USE_MAEMO
-    appdata->search_days = gtk_spin_button_get_value_as_int(
-		    GTK_SPIN_BUTTON(context.spinner));
-#else
-    appdata->search_days = hildon_number_editor_get_value(
-		    HILDON_NUMBER_EDITOR(context.spinner));
-#endif
+    appdata->search_days = number_editor_get_value(context.spinner);
 
     if(check_button_get_active(context.in_finds))
       appdata->search |=  SEARCH_FINDS;
@@ -1486,15 +1543,20 @@ cb_menu_search(GtkWidget *window, gpointer data) {
       printf("Search for %s (flags = %x)...\n", p, appdata->search);
 
 #if !defined(USE_BREAD_CRUMB_TRAIL) && !defined(BCT)
-      gpx_t *found =
-	search_do(appdata, appdata->gpx, p, appdata->search, FALSE);
+
+      if(appdata->cur_gpx)
+	appdata->search_results = 
+	  search_do(appdata, appdata->cur_gpx, p, appdata->search, TRUE);
+      else
+	appdata->search_results = 
+	  search_do(appdata, appdata->gpx, p, appdata->search, FALSE);
 
       /* do search result dialog here ... */
-      cachelist_dialog(appdata, found);
+      cachelist_dialog(appdata, appdata->search_results);
+
 #ifndef USE_STACKABLE_WINDOW
-      search_result_free(found);
-#else
-      appdata->search_results = found;
+      search_result_free(appdata->search_results);
+      appdata->search_results = NULL;
 #endif
 #else
       gpx_t *found = NULL;
@@ -1784,7 +1846,11 @@ HildonAppMenu *menu_create(appdata_t *appdata, int mode) {
     g_signal_connect_after(button, "clicked", 
 			   G_CALLBACK(on_export_clicked), appdata);
     hildon_app_menu_append(menu, GTK_BUTTON(button));
+  }
 
+  /* if search results exist, don't allow another search */
+  if(!appdata->search_results &&  
+     ((mode == MENU_GPXLIST) || (mode == MENU_CACHELIST))) {
     button = gtk_button_new_with_label(_("Search"));
     g_signal_connect_after(button, "clicked", 
 			   G_CALLBACK(cb_menu_search), appdata);
@@ -1984,7 +2050,7 @@ void menu_create(appdata_t *appdata) {
 
 void cleanup(appdata_t *appdata) {
   gconf_save_state(appdata);
-  
+
   gpx_free_all(appdata->gpx);
 
 #ifdef USE_STACKABLE_WINDOW
@@ -1996,12 +2062,12 @@ void cleanup(appdata_t *appdata) {
   icons_free();
   gps_release(appdata);
 
-#ifdef USE_MAEMO
   if(appdata->search_results) {
     printf("freeing pending search\n");
     search_result_free(appdata->search_results);
   }
 
+#ifdef USE_MAEMO
   if(appdata->osso_context)
     osso_deinitialize(appdata->osso_context);
 
@@ -2182,7 +2248,7 @@ crumb_back(gpointer data) {
 
 static void crumb_add(appdata_t *appdata, char *name, int level, 
 		      gpointer user_data) {
-  crumb_t *crumb = malloc(sizeof(crumb_t));
+  crumb_t *crumb = g_new0(crumb_t, 1);
   crumb->level = level;
   crumb->appdata = appdata;
   crumb->data = user_data;
@@ -2262,11 +2328,12 @@ void main_after_settings_redraw(appdata_t *appdata, int flags) {
   }
 
   if(redraw) {
-    GtkWidget *container = appdata->vbox;
 
 #ifdef USE_STACKABLE_WINDOW
     HildonWindowStack *stack = hildon_window_stack_get_default();
-    container = hildon_window_stack_peek(stack);
+    GtkWidget *container = hildon_window_stack_peek(stack);
+#else
+    GtkWidget *container = appdata->vbox;
 #endif
 
     gtk_container_remove(GTK_CONTAINER(container), appdata->cur_view);
@@ -2285,11 +2352,10 @@ void main_after_settings_redraw(appdata_t *appdata, int flags) {
     }
 
 #ifdef USE_STACKABLE_WINDOW
-    if(container != appdata->vbox)
       gtk_container_add(GTK_CONTAINER(container), appdata->cur_view); 
-    else
-#endif
+#else
       gtk_box_pack_start_defaults(GTK_BOX(container), appdata->cur_view);
+#endif
     
     gtk_widget_show_all(container);
   }
@@ -2369,9 +2435,9 @@ int main(int argc, char *argv[]) {
   appdata.clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
   gtk_clipboard_set_can_store(appdata.clipboard, NULL, 0);
 
+#ifndef USE_STACKABLE_WINDOW
   appdata.vbox = gtk_vbox_new(FALSE, 2);
   gtk_container_add(GTK_CONTAINER(appdata.window), appdata.vbox);
-#ifndef USE_STACKABLE_WINDOW
   menu_create(&appdata);
 #else
   hildon_window_set_app_menu(HILDON_WINDOW(appdata.window), 
@@ -2407,7 +2473,11 @@ int main(int argc, char *argv[]) {
   gps_init(&appdata);
 
   appdata.cur_view = gpxlist_create_view_and_model(&appdata, NULL);
+#ifndef USE_STACKABLE_WINDOW
   gtk_box_pack_start_defaults(GTK_BOX(appdata.vbox), appdata.cur_view);
+#else
+  gtk_container_add(GTK_CONTAINER(appdata.window), appdata.cur_view);
+#endif
 
   gtk_widget_show_all(GTK_WIDGET(appdata.window));
   gtk_main();
