@@ -237,6 +237,23 @@ static void map_draw_gpx(appdata_t *appdata, cache_display_t *caches,
   }
 }
 
+static cache_t *cache_search_first(gpx_t *gpx, cache_t *cache) {
+  while(gpx) {
+    cache_t *cur = gpx->cache;
+    while(cur) {
+      /* we do a string comparision to make sure that if copies */
+      /* exist, we get the original one */
+      if(!strcmp(cache->id, cur->id))
+	 return cur;
+
+      cur = cur->next;
+    }
+    gpx = gpx->next;
+  }
+  g_assert(FALSE);
+  return NULL;
+}
+
 /* draw geocaches and set window title */
 static void map_setup(map_context_t *context) {
   char *name = NULL;
@@ -258,7 +275,41 @@ static void map_setup(map_context_t *context) {
     printf("allocated space to handle %d map icons\n", cache_num);
   }
 
-  if(!context->appdata->cur_gpx && !context->appdata->cur_cache) {
+  if(context->appdata->search_results && !context->appdata->cur_cache) {
+    if(context->state != MAP_SEARCH) {
+      printf("map_setup(SEARCH)\n");
+
+      /* clear any pending balloon */
+      context->balloon = NULL;
+      osm_gps_map_osd_clear_balloon (OSM_GPS_MAP(context->widget));
+
+#ifdef OSD_NAV
+      /* no navigation in this mode */
+      osm_gps_map_osd_clear_nav (OSM_GPS_MAP(context->widget));
+#endif
+
+      /* clear all existing cache images */
+      osm_gps_map_clear_images (OSM_GPS_MAP(context->widget));
+      memset(context->caches_displayed, 0, 
+	     (cache_num+1) * sizeof(cache_display_t));
+
+      /* draw search results and all other gpx files are semi-transparent */
+      map_draw_gpx(context->appdata, context->caches_displayed, 
+		   context->widget, context->appdata->search_results, 
+		   NULL, FALSE);
+
+      gpx_t *gpx = context->appdata->gpx;
+      while(gpx) {
+	map_draw_gpx(context->appdata, context->caches_displayed, 
+		     context->widget, gpx, NULL, TRUE);
+
+	gpx = gpx->next;
+      }
+
+      name = g_strdup(_("Search results"));
+      context->state = MAP_SEARCH;
+    }
+  } else if(!context->appdata->cur_gpx && !context->appdata->cur_cache) {
     if(context->state != MAP_ALL) {
       printf("map_setup(ALL)\n");
 
@@ -340,6 +391,10 @@ static void map_setup(map_context_t *context) {
     osm_gps_map_clear_images (OSM_GPS_MAP(context->widget));
     memset(context->caches_displayed, 0, 
 	   (cache_num+1) * sizeof(cache_display_t));
+
+    /* Make sure we really work on the first copy in the list as */
+    /* only this is being used if duplicates exist */
+    cache = cache_search_first(context->appdata->gpx, cache);
     
     /* draw all geocaches and all but selected one are semi-transparent */
     gpx_t *gpx = context->appdata->gpx;
@@ -380,73 +435,12 @@ static void map_setup(map_context_t *context) {
   }
 
   if(name) {
-    char *title = g_strdup_printf(_("Map - %s"), name);
+    gtk_window_set_title(GTK_WINDOW(context->window), name);    
     g_free(name);
-
-    gtk_window_set_title(GTK_WINDOW(context->window), title);
-    
-    g_free(title);
   } else
     printf("map_setup(keep)\n");
 }
 
-static gboolean on_map_configure(GtkWidget *widget,
-				 GdkEventConfigure *event,
-				 map_context_t *context) {
-
-  /* for some reason there's a configure event with 1/1 */
-  /* on diablo. We just ignore this! */
-
-  printf("on_map_configure %d %d\n", 
-	 widget->allocation.width, 
-	 widget->allocation.height);
-
-  if(!context->map_complete && 
-     (widget->allocation.width > 100) &&
-     (widget->allocation.height > 100)) {
-
-    /* setup cache state */
-    map_setup(context);
-
-    /* set default values if they are invalid */
-    if(!context->appdata->map.zoom || 
-       isnan(context->appdata->map.pos.lat) ||
-       isnan(context->appdata->map.pos.lon)) {
-      printf("no valid map position found\n");
-      
-      pos_t *refpos = get_pos(context->appdata);
-      if(refpos && !isnan(refpos->lat) && !isnan(refpos->lon)) {
-	printf("use refpos\n");
-
-	/* use gps position if present */
-	context->appdata->map.pos = *refpos;
-	context->appdata->map.zoom = GPS_DEFAULT_ZOOM;
-      } else {
-	printf("use zero pos\n");
-
-	/* use world map otherwise */
-	context->appdata->map.pos.lat = 0.0;
-	context->appdata->map.pos.lon = 0.0;
-	context->appdata->map.zoom = 1;
-      }
-    }
-    
-    /* jump to initial position */
-    printf("osm_gps_map_set_mapcenter(%f,%f,%d)\n", 
-	   context->appdata->map.pos.lat,
-	   context->appdata->map.pos.lon,
-	   context->appdata->map.zoom);
-	   
-    osm_gps_map_set_mapcenter(OSM_GPS_MAP(context->widget), 
-			      context->appdata->map.pos.lat, 
-			      context->appdata->map.pos.lon, 
-			      context->appdata->map.zoom);
-    context->map_complete = TRUE;
-  }
-
-  return FALSE;
-}
-
 static void
 map_cachelist_nearest(cache_t *cache, pos_t *pos, 
 		      cache_t **result, float *distance) {
@@ -470,8 +464,12 @@ map_cachelist_nearest(cache_t *cache, pos_t *pos,
 static cache_t *map_closest(map_context_t *context, pos_t *pos) {
   cache_t *result = NULL;
   float distance = NAN;
-  
-  if(!context->appdata->cur_gpx && !context->appdata->cur_cache) {
+
+  if(context->appdata->search_results && !context->appdata->cur_cache) {
+    /* search search results */
+    map_cachelist_nearest(context->appdata->search_results->cache, 
+			  pos, &result, &distance);
+  } else if(!context->appdata->cur_gpx && !context->appdata->cur_cache) {
     /* search all geocaches */
     gpx_t *gpx = context->appdata->gpx;
     while(gpx) {
@@ -479,10 +477,14 @@ static cache_t *map_closest(map_context_t *context, pos_t *pos) {
       gpx = gpx->next;
     }
   } else if(context->appdata->cur_gpx) {
+    /* search in current gpx file */
     map_cachelist_nearest(context->appdata->cur_gpx->cache, 
 			  pos, &result, &distance);
-  } else
-    result = context->appdata->cur_gpx->cache;
+  } else 
+    result = context->appdata->cur_cache;
+
+  /* make sure this is the first hit */
+  result = cache_search_first(context->appdata->gpx, result);
 
   return result;
 }
@@ -706,17 +708,21 @@ balloon_cb(osm_gps_map_balloon_event_t *event, gpointer data) {
     printf("click %s event at %d %d\n", 
 	   event->data.click.down?"down":"up",
 	   event->data.click.x, event->data.click.y);
-
+    
     /* make the main screen jump to that cache */
     if(!event->data.click.down) {
       if(context->appdata->cur_cache) {
 	printf("ERROR: no current cache should be visible!\n");
       } else {
 	gpx_t *is_in = NULL;
+	
+	if(context->appdata->search_results) {
+	  printf("click while in \"search results\" view\n");
 
-	if(!context->appdata->cur_gpx) {
+	  is_in = context->appdata->search_results;
+	} else if(!context->appdata->cur_gpx) {
 	  printf("click while in \"all\" view\n");
-
+	  
 	  /* we first need to figure out which gpx file this cache */
 	  /* is in so we can open it first */
 	  gpx_t *gpx = context->appdata->gpx;
@@ -732,19 +738,24 @@ balloon_cb(osm_gps_map_balloon_event_t *event, gpointer data) {
 	  
 	  if(is_in)
 	    gpxlist_goto_cachelist(context->appdata, is_in);
-
-	} else 
+	  
+	} else {
+	  printf("click while in \"cachelist\" view\n");
+	
 	  /* the simple case: there already is an open gpx file and */
 	  /* we just jump into the "cache" view */
 	  is_in = context->appdata->cur_gpx;
-
+	}
+	
 	if(is_in) {
 	  printf("selecting %s in %s\n", 
 		 cache->id,
+		 context->appdata->search_results?
+		 context->appdata->search_results->name:
 		 context->appdata->cur_gpx->name);
-
+	  
 	  cachelist_goto_cache(context->appdata, cache);
-
+	  
 	  /* give focus to main screen (important for maemo) */
 	  printf("raising main window\n");
 	  gtk_window_present(GTK_WINDOW(context->appdata->window));
@@ -922,7 +933,6 @@ void map(appdata_t *appdata) {
   printf("allocated new context at %p\n", context);
 
   context->appdata = appdata;
-  context->map_complete = FALSE;
   context->state = MAP_NONE;
 
   /* cleanup old (pre 0.8.7) path if it exists */
@@ -967,6 +977,34 @@ void map(appdata_t *appdata) {
 
   osm_gps_map_osd_classic_init(OSM_GPS_MAP(context->widget));
 
+  /* set default values if they are invalid */
+  if(!context->appdata->map.zoom || 
+     isnan(context->appdata->map.pos.lat) ||
+     isnan(context->appdata->map.pos.lon)) {
+    printf("no valid map position found\n");
+    
+    pos_t *refpos = get_pos(context->appdata);
+    if(refpos && !isnan(refpos->lat) && !isnan(refpos->lon)) {
+      printf("use refpos\n");
+      
+	/* use gps position if present */
+      context->appdata->map.pos = *refpos;
+      context->appdata->map.zoom = GPS_DEFAULT_ZOOM;
+    } else {
+      printf("use zero pos\n");
+      
+      /* use world map otherwise */
+      context->appdata->map.pos.lat = 0.0;
+      context->appdata->map.pos.lon = 0.0;
+      context->appdata->map.zoom = 1;
+    }
+  }
+  
+  osm_gps_map_set_mapcenter(OSM_GPS_MAP(context->widget), 
+			    context->appdata->map.pos.lat, 
+			    context->appdata->map.pos.lon, 
+			    context->appdata->map.zoom);
+  
 #ifdef USE_MAEMO
   /* we don't use a stackable window here on fremantle, since */
   /* this leaves the main window independent from the map and */
@@ -991,9 +1029,6 @@ void map(appdata_t *appdata) {
   g_signal_connect(G_OBJECT(context->widget), "focus-out-event", 
 		   G_CALLBACK(on_focus_out), context);
 
-  g_signal_connect(G_OBJECT(context->widget), "configure-event",
-		   G_CALLBACK(on_map_configure), context);
-
   g_signal_connect(G_OBJECT(context->widget), "button-press-event",
                    G_CALLBACK(on_map_button_press_event), context);
 
@@ -1015,4 +1050,7 @@ void map(appdata_t *appdata) {
 
   gtk_container_add(GTK_CONTAINER(context->window), context->widget); 
   gtk_widget_show_all(GTK_WIDGET(context->window));
+
+  /* setup cache state */
+  map_setup(context);
 }
