@@ -51,6 +51,17 @@ float roundf(float x);
 #define COORDINATE_PICKER
 #endif
 
+static const char *decimal_point(void) {
+  static const char *point = ".";
+
+  struct lconv *lconv = localeconv();
+
+  if(lconv && lconv->decimal_point)
+    return lconv->decimal_point;
+
+  return point;
+}
+
 char strlastchr(char *str) {
   return str[strlen(str)]-1;
 }
@@ -568,7 +579,8 @@ static gint on_lat_picker_button_press(GtkWidget *button,
 		minw[0] = digit_picker_create(0,5, min/10));
     gtk_box_pack_start_defaults(GTK_BOX(hbox), 
 		minw[1] = digit_picker_create(0,9, min%10));
-    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("."), FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(decimal_point()), 
+		       FALSE, FALSE, 0);
 
     gtk_box_pack_start_defaults(GTK_BOX(hbox), 
 		minfracw[0] = digit_picker_create(0,9, minfrac/100));
@@ -612,6 +624,40 @@ static gint on_lat_picker_button_press(GtkWidget *button,
 }
 #endif
 
+/* whatever there is in the entry: if it's illegal make it */
+#define ILLEGAL_LAT  "X --° --%s---'"
+#define ILLEGAL_LON  "X ---° --%s---'"
+
+static void pos_lat_check(GtkWidget *widget) {  
+  if(isnan(lat_entry_get(widget))) {
+    char *str = g_strdup_printf(ILLEGAL_LAT, decimal_point());
+
+#ifndef COORDINATE_PICKER
+    mark(widget, FALSE);
+    gtk_entry_set_text(GTK_ENTRY(widget), str);
+#else
+    gtk_button_set_label(GTK_BUTTON(widget), str);
+#endif
+
+    g_free(str);
+  }    
+}
+
+static void pos_lon_check(GtkWidget *widget) {  
+  if(isnan(lon_entry_get(widget))) {
+    char *str = g_strdup_printf(ILLEGAL_LON, decimal_point());
+
+#ifndef COORDINATE_PICKER
+    mark(widget, FALSE);
+    gtk_entry_set_text(GTK_ENTRY(widget), str);
+#else
+    gtk_button_set_label(GTK_BUTTON(widget), str);
+#endif
+
+    g_free(str);
+  }    
+}
+
 /* a entry that is colored red when being "active" */
 GtkWidget *lat_entry_new(float lat) {
   char str[32];
@@ -636,6 +682,7 @@ GtkWidget *lat_entry_new(float lat) {
   		     (GtkSignalFunc)on_lat_picker_button_press, NULL);
 #endif
   
+  pos_lat_check(widget);
   return widget;
 }
 
@@ -688,7 +735,8 @@ static gint on_lon_picker_button_press(GtkWidget *button,
 		minw[0] = digit_picker_create(0,5, min/10));
     gtk_box_pack_start_defaults(GTK_BOX(hbox), 
 		minw[1] = digit_picker_create(0,9, min%10));
-    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("."), FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(decimal_point()), 
+		       FALSE, FALSE, 0);
 
     gtk_box_pack_start_defaults(GTK_BOX(hbox), 
 		minfracw[0] = digit_picker_create(0,9, minfrac/100));
@@ -754,6 +802,7 @@ GtkWidget *lon_entry_new(float lon) {
   		     (GtkSignalFunc)on_lon_picker_button_press, NULL);
 #endif
 
+  pos_lon_check(widget);
   return widget;
 }
 
@@ -786,6 +835,7 @@ void lat_entry_set(GtkWidget *widget, float lat) {
   g_object_set_data(G_OBJECT(widget), "latitude", (gpointer)lat_int); 
   g_signal_emit_by_name(widget, "changed");
 #endif
+  pos_lat_check(widget);
 }
 
 void lon_entry_set(GtkWidget *widget, float lon) {
@@ -799,6 +849,7 @@ void lon_entry_set(GtkWidget *widget, float lon) {
   g_object_set_data(G_OBJECT(widget), "longitude", (gpointer)lon_int); 
   g_signal_emit_by_name(widget, "changed");
 #endif
+  pos_lon_check(widget);
 }
 
 void lat_label_set(GtkWidget *widget, float lat) {
@@ -2110,4 +2161,97 @@ GtkWidget *export_file(char *sel_title, char **filename) {
 #endif
 }
 
+static int number_editor_get(GtkWidget *widget) {
+  GtkWidget *entry = g_object_get_data(G_OBJECT(widget), "entry");
+  g_assert(entry);
+  const char *str = gtk_entry_get_text(GTK_ENTRY(entry));
+  if(!str) return 0;
+  return atoi(str);
+}
 
+static void number_editor_set(GtkWidget *widget, int num) {
+  int min = (int)g_object_get_data(G_OBJECT(widget), "min");
+  int max = (int)g_object_get_data(G_OBJECT(widget), "max");
+
+  if(num < min) num = min;
+  if(num > max) num = max;
+
+  GtkWidget *btn_inc = 
+    (GtkWidget*)g_object_get_data(G_OBJECT(widget), "btn_inc");
+  gtk_widget_set_sensitive(btn_inc, num < max);
+
+  GtkWidget *btn_dec = 
+    (GtkWidget*)g_object_get_data(G_OBJECT(widget), "btn_dec");
+  gtk_widget_set_sensitive(btn_dec, num > min);
+
+  GtkWidget *entry = g_object_get_data(G_OBJECT(widget), "entry");
+  g_assert(entry);
+  char *str = g_strdup_printf("%d", num);
+  gtk_entry_set_text(GTK_ENTRY(entry), str);
+  g_free(str);
+}
+
+static void on_number_decrease(GtkWidget *widget, gpointer data) {
+  number_editor_set(GTK_WIDGET(data), 
+    number_editor_get(GTK_WIDGET(data)) - 1);
+}
+
+static void on_number_increase(GtkWidget *widget, gpointer data) {
+  number_editor_set(GTK_WIDGET(data), 
+    number_editor_get(GTK_WIDGET(data)) + 1);
+}
+
+static void on_number_changed(GtkWidget *widget, gpointer data) {
+  number_editor_set(GTK_WIDGET(data), 
+    number_editor_get(GTK_WIDGET(data)));
+}
+
+GtkWidget *number_editor_new(int current, int min, int max) {
+#ifndef USE_MAEMO
+  GtkObject *adj = gtk_adjustment_new(current, min, max, 1, 10, 10);
+  return gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
+#elif MAEMO_VERSION_MAJOR < 5
+  GtkWidget *widget = hildon_number_editor_new(min, max);
+  hildon_number_editor_set_value(HILDON_NUMBER_EDITOR(widget), current);
+  return widget;
+#else
+  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
+  g_object_set_data(G_OBJECT(hbox), "min", (gpointer)min);
+  g_object_set_data(G_OBJECT(hbox), "max", (gpointer)max);
+
+  GtkWidget *btn_dec = button_new_with_label("  -  ");
+  g_object_set_data(G_OBJECT(hbox), "btn_dec", btn_dec);
+  gtk_signal_connect(GTK_OBJECT(btn_dec), "clicked",
+  		     (GtkSignalFunc)on_number_decrease, hbox);
+  gtk_box_pack_start(GTK_BOX(hbox), btn_dec, FALSE, FALSE, 0);
+
+  GtkWidget *entry = hildon_entry_new(HILDON_SIZE_AUTO);
+  g_object_set_data(G_OBJECT(hbox), "entry", entry);
+  g_signal_connect(G_OBJECT(entry), "changed",
+		   G_CALLBACK(on_number_changed), hbox);
+  gtk_entry_set_width_chars(GTK_ENTRY(entry), 2);
+  hildon_gtk_entry_set_input_mode(GTK_ENTRY(entry), 
+				  HILDON_GTK_INPUT_MODE_NUMERIC);
+  gtk_box_pack_start_defaults(GTK_BOX(hbox), entry);
+
+  GtkWidget *btn_inc = button_new_with_label("  +  ");
+  g_object_set_data(G_OBJECT(hbox), "btn_inc", btn_inc);
+  gtk_signal_connect(GTK_OBJECT(btn_inc), "clicked",
+  		     (GtkSignalFunc)on_number_increase, hbox);
+  gtk_box_pack_start(GTK_BOX(hbox), btn_inc, FALSE, FALSE, 0);
+
+  number_editor_set(hbox, current);
+
+  return hbox;
+#endif
+}
+
+int number_editor_get_value(GtkWidget *widget) {
+#ifndef USE_MAEMO
+  return gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+#elif MAEMO_VERSION_MAJOR < 5
+  return hildon_number_editor_get_value(HILDON_NUMBER_EDITOR(widget));
+#else
+  return number_editor_get(widget);
+#endif
+}
