@@ -128,4 +128,7 @@ GtkWidget *button_new_with_label(char *label);
 
 GtkWidget *export_file(char *sel_title, char **filename);
 
+GtkWidget *number_editor_new(int current, int min, int max);
+int number_editor_get_value(GtkWidget *widget);
+
 #endif // MISC_H
