@@ -311,26 +311,12 @@ static gboolean update(gpointer data) {
   g_free(mup);
 #endif
   
-  pos_lat_str(str, sizeof(str), context->avg.lat);
-  gtk_label_set_text(GTK_LABEL(context->lat_label), str);
-  pos_lon_str(str, sizeof(str), context->avg.lon);
-  gtk_label_set_text(GTK_LABEL(context->lon_label), str);
+  lat_label_set(context->lat_label, context->avg.lat);
+  lon_label_set(context->lon_label, context->avg.lon);
 
   return TRUE;   // fire again
 }
 
-#ifdef ENABLE_MAEMO_MAPPER
-static void on_mm_export_clicked(GtkButton *button, gpointer data) {
-  pp_context_t *context = (pp_context_t*)data;
-
-  if(!context->avg.lat || !context->avg.lon)
-    return;
-
-  pos_t pos = { context->avg.lat, context->avg.lon } ;
-  dbus_mm_set_position(context->appdata, &pos);
-}
-#endif
-
 static void on_copy_clicked(GtkButton *button, gpointer data) {
   pp_context_t *context = (pp_context_t*)data;
   char str[64];
@@ -415,15 +401,6 @@ void precise_position(appdata_t *appdata) {
 
   context->handler_id = gtk_timeout_add(1000, update, context);
 
-#ifdef ENABLE_MAEMO_MAPPER
-  /* ------------- maemo mapper button ---------------- */  
-  GtkWidget *button = gtk_button_new();
-  gtk_button_set_image(GTK_BUTTON(button), icon_get_widget(ICON_MISC, 0));
-  gtk_signal_connect(GTK_OBJECT(button), "clicked",
-  		     (GtkSignalFunc)on_mm_export_clicked, context);
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area), button);
-#endif
-
   update(context);
   gtk_widget_show_all(dialog);
   gtk_dialog_run(GTK_DIALOG(dialog));
