@@ -666,6 +666,9 @@ void cb_menu_settings(GtkWidget *window, gpointer data) {
     appdata->imperial = 
       check_button_get_active(cbox_imperial);
 
+    /* make sure GPS change is honoured by gps routines */
+    gps_change_state(appdata);
+
     if(appdata->username) {
       g_free(appdata->username);
       appdata->username = NULL;
